"""
smarty.utils
~~~~~~~~~~~~

This module implements all utility methods.
"""

import json
import random

import names
import requests


BASE = "https://api.smartrecruiters.com"
SOURCE_DATA = json.loads(open("smarty_py/resources/source_data.json", "r").read())


def call(endpoint, method="GET", expected_code=200, **kwargs):
    """ Call a given endpoint.

    :param endpoint: a `str` indicating the target endpoint.
    :param method: a `str` indicating the HTTP method to use.
    """
    url = "".join([BASE, endpoint])

    for key in kwargs:
        if key not in ("headers", "data", "params"):
            kwargs.pop(key)
    else:
        response = getattr(requests, method.lower())(url, **kwargs)

    if response.status_code == expected_code:
        try:
            return json.loads(response.text)
        except json.JSONDecodeError:
            return response.text
    else:
        raise Exception(f"Error: {response.status_code}")


def create_candidate(instance):
    """ Generates candidate object for API submission
    
    :return candidate_json: a JSON object of candidate data.
    """
    if instance["type"] == "US":
        companies = SOURCE_DATA["companies_us"]
        locations = SOURCE_DATA["locations_us"]
    elif instance["type"] == "EU":
        companies = SOURCE_DATA["companies_eu"]
        locations = SOURCE_DATA["locations_eu"]

    first_name = names.get_first_name()
    last_name = names.get_last_name()
    location = random.choice(locations)
    uni_start = random.randint(1980, 2000)
    work_start = uni_start + random.randint(3, 6)
    source = random.choice(SOURCE_DATA["sources"])
    candidate = {
        "firstName": first_name,
        "lastName": last_name,
        "email": "".join(
            [
                first_name,
                ".",
                last_name,
                str(random.randint(100, 9999)),
                "@mailinator.com",
            ]
        ),
        "phone": str(random.randint(1000000, 9999999)),
        "location": {
            "country": location[0],
            "countryCode": location[1],
            "regionCode": "",
            "region": location[4],
            "city": location[3],
            "address": "",
            "postalCode": "",
        },
        "web": {
            "skype": "",
            "linkedin": "",
            "facebook": "",
            "twitter": "",
            "website": "",
        },
        "tags": [random.choice(SOURCE_DATA["tags"]) for _ in range(3)],
        "education": [
            {
                "institution": random.choice(SOURCE_DATA["schools"]),
                "degree": "B.A.",
                "major": random.choice(SOURCE_DATA["majors"]),
                "current": False,
                "location": "",
                "startDate": str(uni_start),
                "endDate": str(uni_start + random.randint(4, 6)),
            }
        ],
        "experience": [
            {
                "title": random.choice(SOURCE_DATA["titles"]),
                "company": random.choice(companies),
                "current": True,
                "startDate": str(work_start),
                "endDate": str(work_start + 2),
                "location": "",
                "description": ", ".join(
                    [random.choice(SOURCE_DATA["descriptions"]) for _ in range(2)]
                ),
            },
            {
                "title": random.choice(SOURCE_DATA["titles"]),
                "company": random.choice(companies),
                "current": True,
                "startDate": str(work_start + 2),
                "endDate": str(work_start + 4),
                "location": "",
                "description": ", ".join(
                    [random.choice(SOURCE_DATA["descriptions"]) for _ in range(2)]
                ),
            },
            {
                "title": random.choice(SOURCE_DATA["titles"]),
                "company": random.choice(companies),
                "current": True,
                "startDate": str(work_start + 4),
                "endDate": str(work_start + 6),
                "location": "",
                "description": ", ".join(
                    [random.choice(SOURCE_DATA["descriptions"]) for _ in range(2)]
                ),
            },
        ],
        "sourceDetails": {"sourceTypeId": source[0], "sourceId": source[1]},
    }

    return candidate


def get_instance(token):
    """ Gets and returns instance data from the configuration API.

    :param token: a `str` SmartToken.
    :return instance: a `dict` of instance information.
    """
    endpoint = "/configuration/company"
    headers = {"X-SmartToken": token}

    response = call(endpoint, headers=headers)

    instance = response
    instance["type"] = (
        "US" if instance["location"]["country"] == "United States" else "EU"
    )

    return instance


def get_tokens():
    """ Gets and returns all saved SmartTokens from tokens.txt

    :return tokens: a `list` of tokens.
    """
    try:
        token_file = open("smarty_py/resources/tokens.txt", "r")
    except FileNotFoundError:
        open("smarty_py/resources/tokens.txt", "w+")
        token_file = open("smarty_py/resources/tokens.txt", "r")

    tokens = []
    for line in token_file.readlines():
        vals = line.split(" | ")
        tokens.append([vals[0], vals[1]])

    return tokens


def save_token(token, name):
    """ Saves a token and its instance name in tokens.txt.

    :param token: a `str` SmartToken.
    :param name: a `str` name for the token.
    """
    token_file = open("smarty_py/resources/tokens.txt", "a+")
    token_file.write("".join([name, " | ", token]))



"""
srapi.candidate
~~~~~~~~~~~~~~~

This module implements the Candidate API client.
"""

import json

from smarty_py.utils import call


class CandidateAPI:
    """ Partial hardcoded Candidate API endpoint access. """

    def __init__(self, token):
        """ Initialize the Candidate API.

        :param token: a SmartRecruiters API token
        """
        self.headers = {"X-SmartToken": token}

    def search(self, **kwargs):
        """ Search for candidates. """
        endpoint = "/candidates"
        method = "get"

        response = call(endpoint, method, headers=self.headers, params=kwargs)

        return response

    def create(self, **kwargs):
        """ Create a candidate.

        POST /candidates
        """
        endpoint = "/candidates"
        method = "post"
        data = json.dumps(kwargs)

        headers = self.headers
        headers["Content-Type"] = "application/json"

        response = call(endpoint, method, expected_code=201, headers=headers, data=data)

        return response

    def create_and_assign(self, job_id, **kwargs):
        """ Create a candidate and assign to a job.

        POST /jobs/{jobId}/candidates
        """
        endpoint = f"/jobs/{job_id}/candidates"
        method = "post"
        data = json.dumps(kwargs)

        headers = self.headers
        headers["Content-Type"] = "application/json"

        response = call(endpoint, method, expected_code=201, headers=headers, data=data)

        return response

    def get(self, cid):
        """ Get a candidate's details.

        GET /candidates/{cid}
        """
        endpoint = f"/candidates/{cid}"
        method = "get"

        response = call(endpoint, method, headers=self.headers)

        return response

    def delete(self, cid):
        """ Delete a candidate.

        DELETE /candidates/{cid}
        """
        endpoint = f"/candidates/{cid}"
        method = "delete"

        response = call(endpoint, method, headers=self.headers)

        return response

    def update(self, cid):
        """ Update a candidate.

        PATCH /candidates/{cid}
        """
        endpoint = "/candidates/{}".format(cid)
        method = "patch"

        response = call(endpoint, method, headers=self.headers)

        return response



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['smarty_py', 'smarty_py.api', 'smarty_py.resources']

package_data = \
{'': ['*']}

install_requires = \
['colorama>=0.4.3,<0.5.0', 'names>=0.3.0,<0.4.0', 'requests>=2.22.0,<3.0.0']

setup_kwargs = {
    'name': 'smarty-py',
    'version': '0.1.2',
    'description': 'A CLI for SmartRecruiters solutions consulting.',
    'long_description': None,
    'author': 'Elliott Maguire',
    'author_email': 'e.maguire@smartrecruiters.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""
smarty
~~~~~~

This is a instance management CLI for Smartrecruiters
solutions consulting.
"""

import json
import os
import sys
import time

from smarty_py import utils
from smarty_py.api import CandidateAPI

from colorama import init

init()
from colorama import Fore, Back, Style

clear = lambda: os.system("cls" if os.name == "nt" else "clear")

BASE_URL = "https://api.smartrecruiters.com"


def main():
    clear()

    print(Fore.GREEN + "Welcome to smarty.py v2.0" + Fore.RESET)

    token = input(
        "If you have a token saved, press enter to choose. "
        "Otherwise, please enter your SmartToken to begin:\n"
    )
    if token:
        try:
            instance = utils.get_instance(token)
        except:  # handle auth error
            print(
                Fore.RED
                + "Error 01 - Invalid SmartToken; please try again."
                + Fore.RESET
            )

            time.sleep(2)

            main()
        else:  # store new token
            instance["token"] = token
            utils.save_token(instance["token"], instance["name"])

            menu(instance)
    else:
        tokens = utils.get_tokens()
        print(Fore.CYAN + "Select a token from the following:" + Fore.RESET)
        for i, pair in enumerate(tokens):
            print(Fore.LIGHTYELLOW_EX + f"{i + 1}. {pair[0]} | {pair[1]}" + Fore.RESET)

        selection = input("\nSelection: ")

        try:
            instance = utils.get_instance(tokens[int(selection) - 1][1])
        except:  # handle auth error
            print(
                Fore.RED
                + "Error 01 - Invalid SmartToken; please try again."
                + Fore.RESET
            )
            time.sleep(2)

            main()
        else:
            instance["token"] = tokens[int(selection) - 1][1]

            menu(instance)


def menu(instance):
    """ CLI: Main Menu

    Direct user to other CLI methods.

    :param instance: a `dict` of instance information.
    """
    clear()

    print(
        Fore.GREEN
        + f"Current Instance: {instance['name']} {instance['type']}"
        + Fore.RESET
    )
    print(Fore.CYAN + "Main Menu - Select an Action" + Fore.RESET)
    print(
        Fore.LIGHTYELLOW_EX
        + """
        1. Manage Candidates
        2. Exit
        """
        + Fore.RESET
    )

    selection = input("Selection: ")
    if selection == "1":
        manage_candidates(instance)
    elif selection == "2":
        clear()

        print(Fore.GREEN + "Thanks for using Smarty Py!" + Fore.RESET)

        sys.exit()
    else:
        print("Invalid input. Please try again.")

        time.sleep(1)

        menu(instance)


def manage_candidates(instance):
    """ CLI: Candidates Menu

    List options regarding candidate management,
    each tied to one or more smartlib functions.

    Arguments:
        instance -- a dictionary of SR instance information
    
    """
    print("Loading CandidateAPI client...")
    api = CandidateAPI(instance["token"])

    clear()

    print(Fore.GREEN + "Current Instance: %s" % (instance["name"]) + Fore.RESET)
    print(Fore.CYAN + "Departments Menu - Select an Action" + Fore.RESET)
    print(
        Fore.LIGHTYELLOW_EX
        + """
        1. Search Candidates
        2. Add Candidate
        3. Add Candidate and Assign to Job
        4. Get Candidate Info
        5. Return to Main Menu
        """
        + Fore.RESET
    )

    selection = input("Selection: ")
    if selection == "1":  # search candidates
        clear()

        print(Fore.GREEN + "Current Instance: %s" % (instance["name"]) + Fore.RESET)
        print(Fore.CYAN + "Candidate Search" + Fore.RESET)

        q = input("Search Term: ")
        l = input("Limit: ")
        o = input("Offset: ")
        try:
            results = api.search(params={"q": q, "limit": l, "offset": o})
        except:
            print(
                Fore.RED
                + "\nThere are no candidates matching that search term."
                + Fore.RESET
            )
        else:
            if results:
                print(
                    Fore.CYAN
                    + f"\n{results['totalFound']} Total | Search Results:"
                    + Fore.RESET
                )

                for r in results["content"]:
                    print(
                        Fore.MAGENTA
                        + f"{r['firstName']} {r['lastName']} | {r['id']}"
                        + Fore.RESET
                    )
            else:
                print(
                    Fore.RED
                    + "\nThere are no candidates matching that search term."
                    + Fore.RESET
                )

        action = input("Press enter to return to Candidate Management.")
        if action == "":
            manage_candidates(instance)

    elif selection == "2":  # add candidates
        clear()

        print(Fore.GREEN + "Current Instance: %s" % (instance["name"]) + Fore.RESET)
        print(Fore.CYAN + "Candidate Creation" + Fore.RESET)

        number = input("Number of candidates to add: ")
        for _ in range(int(number)):
            candidate = utils.create_candidate(instance)

            try:
                response = api.create(**candidate)
            except:
                print(
                    Fore.RED + "There was an error adding that candidate." + Fore.RESET
                )
            else:
                print(
                    Fore.YELLOW
                    + f"Candidate added: {response['firstName']} {response['lastName']}"
                    + Fore.RESET
                )

        action = input("Press enter to return to Candidate Management.")
        if action == "":
            manage_candidates(instance)

    elif selection == "3":  # add and assign candidate to job
        clear()

        print(Fore.GREEN + "Current Instance: %s" % (instance["name"]) + Fore.RESET)
        print(Fore.CYAN + "Candidate Creation and Assignment" + Fore.RESET)

        job_id = input("Job ID: ")
        number = input("Number of candidates to add and assign: ")
        for _ in range(int(number)):
            candidate = utils.create_candidate(instance)

            try:
                response = api.create_and_assign(job_id, **candidate)
            except:
                print(
                    Fore.RED
                    + "There was an error adding and assigning that candidate."
                    + Fore.RESET
                )
            else:
                print(
                    Fore.YELLOW
                    + f"Candidate added: {response['firstName']} {response['lastName']}"
                    + Fore.RESET
                )

        action = input("Press enter to return to Candidate Management.")
        if action == "":
            manage_candidates(instance)

    elif selection == "4":  # get candidate details
        clear()

        print(Fore.GREEN + "Current Instance: %s" % (instance["name"]) + Fore.RESET)
        print(Fore.CYAN + "Candidate Info" + Fore.RESET)

        candidate_id = input("Candidate ID: ")

        try:
            response = api.get(candidate_id)
        except ValueError:
            print(Fore.RED + "\nInvalid input; try again." + Fore.RESET)
        except:
            print(Fore.RED + "\nThat ID does not match a candidate." + Fore.RESET)
        else:
            print(
                Fore.YELLOW
                + f"""
            {response['firstName']} {response['lastName']}
            {response['email']}
            https://www.smartrecruiters.com/app/people/candidates/{response['id']}
            """
                + Fore.RESET
            )

        action = input("Press enter to return to Candidate Management.")
        if action == "":
            manage_candidates(instance)

    elif selection == "5":
        menu(instance)

    else:
        print("Invalid input. Please try again.")

        time.sleep(1)

        manage_candidates(instance)


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        clear()

        print(Fore.GREEN + "Thanks for using smarty.py!" + Fore.RESET)
